--[[

						>> PLANETOID <<

	First explorations in Lua / Love2D
	by Daedalus Young

	Stars and galaxy photo downloaded from
		http://www.spacetelescope.org/images/heic1112a/
		CC-BY 3.0 Unported ESA/Hubble

	Game source code and other artwork ("the work") released under a
		Creative Commons Attribution 3.0 license

	You are free:
		To Share- to copy, distribute and transmit the work
		To Remix- to adapt the work
		To make commercial use of the work

	Under the following condition:
		Attribution- You must attribute the work in the manner
			specified by the author or licensor (but not in any
			way that suggests that they endorse you or your use
			of the work).

	With the understanding that:
		Waiver- Any of the above conditions can be waived if
			you get permission from the copyright holder.
		Public Domain- Where the work or any of its elements
			is in the public domain under applicable law, that
			status is in no way affected by the license.
		Other Rights- In no way are any of the following rights
			affected by the license:
			* Your fair dealing or fair use rights, or other
				applicable copyright exceptions and limitations;
			* The author's moral rights;
			* Rights other persons may have either in the work
				itself or in how the work is used, such as
				publicity or privacy rights.

	NOTICE - For any reuse or distribution, you must make clear
		to others the license terms of this work. The best way to
		do this is with a link to the web page
		http://creativecommons.org/licenses/by/3.0/.

--]]

function love.load()
	waitforselect = 0
	countforselect = 0
	showdebug = false
	width = love.graphics.getWidth()
	height = love.graphics.getHeight()
	gamestate = 0
	maxmenu = 3
	elapsed = 0
	selected = 1
	pressedkey = ""
	score = 0
	modes = love.graphics.getModes()
	table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)
	rem = {}
	for z = 1, #modes do
		if modes[z]['width'] < 640 or modes[z]['width'] > 1280 or modes[z]['height'] < 360 or modes[z]['height'] > 720 then
			table.insert(rem, z)
		end
	end
    for z = #rem, 1, -1 do
    	table.remove(modes, rem[z])
    end
	for z = 1, #modes do
		if modes[z]['width'] > 700 and modes[z]['width'] < 850 and modes[z]['height'] > 350 and modes[z]['height'] < 550 then
			newmode = z
		end
	end
	if newmode then
		width = modes[newmode]['width']
		height = modes[newmode]['height']
	end
	love.graphics.setMode(width, height)
	sizeselect = newmode
	direction = 0
	dirspeed = 0
	stars = {}
	updnext = 2
	move = {false, false, false, false}
	ranvalue = {}
	for x = 0, 1023 do
		ranvalue[x] = (math.random() * 3) - 1
	end
	ranvalue[1024] = ranvalue[0]

	--sound
	sound = {	["bloop"] = love.audio.newSource('snd/bloop.ogg', "static"),
				["cursor"] = love.audio.newSource('snd/cursor.ogg', "static"),
				["explode"] = love.audio.newSource('snd/explode.ogg', "static"),
				["gameover"] = love.audio.newSource('snd/gameover.ogg', "static"),
				["hit"] = love.audio.newSource('snd/hit.ogg', "static"),
				["select"] = love.audio.newSource('snd/select.ogg', "static"),
				["shoot"] = love.audio.newSource('snd/shoot.ogg', "static"),
				["spawn"] = {	love.audio.newSource('snd/spawn1.ogg', "static"),
								love.audio.newSource('snd/spawn2.ogg', "static"),
								love.audio.newSource('snd/spawn3.ogg', "static"),
								love.audio.newSource('snd/spawn4.ogg', "static") } }

	--white font
	local fimg = love.graphics.newImage("img/dyspacefont.png")
	fimg:setFilter("nearest", "nearest")
	font = love.graphics.newImageFont(fimg, " !?-=+_<>,.:;[]()*0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\"/#")

	--red font
	fimg = love.graphics.newImage("img/dyspacefontr.png")
	fimg:setFilter("nearest", "nearest")
	fontred = love.graphics.newImageFont(fimg, " !?-=+_<>,.:;[]()*0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\"/#")

	--green font
	fimg = love.graphics.newImage("img/dyspacefontg.png")
	fimg:setFilter("nearest", "nearest")
	fontgreen = love.graphics.newImageFont(fimg, " !?-=+_<>,.:;[]()*0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\"/#")

	--blue font
	fimg = love.graphics.newImage("img/dyspacefontb.png")
	fimg:setFilter("nearest", "nearest")
	fontblue = love.graphics.newImageFont(fimg, " !?-=+_<>,.:;[]()*0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'\"/#")
	love.graphics.setFont(font)
	love.graphics.setBackgroundColor(0, 0, 0)
	bg = love.graphics.newImage('img/hubble.jpg')
	sprites = love.graphics.newImage('img/sprites.png')
	bullets = love.graphics.newImage('img/bullets.png')
	spritecounter = 0
	spacecounter = 5
	enemy = {}
	xmove = 0
	ymove = 0
	waitforenemy = 0
	enbullet = love.graphics.newQuad(0, 16, 16, 16, 16, 32, bullets)
	plbullet = love.graphics.newQuad(0, 0, 16, 16, 16, 32, bullets)
	pshots = {}
	eshots = {}
	player = love.graphics.newQuad(0, 0, 120, 40, 480, 200, sprites)
	ufo = love.graphics.newQuad(0, 120, 120, 40, 480, 200, sprites)
	playerx = math.floor(width / 10)
	playery = (height / 2) - 20
	text, textlen = love.filesystem.read("text.txt")
	scrolltext = "   "
	startpos = 100 - (width/12.8)
	position = startpos
	textatpixel = 0
	scrollspeed = 100
	minwaittime = 4.5
	health = 3
	hillinit()
end

function hillinit()
	stars = {}
	maxstars = math.floor((width * height) / 6000)
	for x = 1, maxstars do
		starfield(x)
	end
	hillwidth = width * 1.1
	hilldata = love.image.newImageData(hillwidth + 1, math.floor(height / 5))
	hills = love.graphics.newImage(hilldata)
	ground = love.image.newImageData('img/ground.png')
end

function newenemy()
	z = #enemy + 1
	enemy[z] = {['sprite'] = love.graphics.newQuad(0, 120, 120, 40, 480, 200, sprites), ['x'] = width, ['y'] = math.random(height / 8, height - (height / 3.3)), ['timer'] = math.random()}
	local rsound = math.random(1,4)
	sound['spawn'][rsound]:setPitch(1 + (math.random() * 0.6) - 0.3)
	love.audio.play(sound['spawn'][rsound])
end

function love.focus(f)
	if gamemode == 1 then
		if not f then paused = true end
	end
end

function love.update(dt)
	if gamestate == 0 then
		--menu
		dymenu()
		if waitforselect > 0 then
			countforselect = countforselect + dt
			if countforselect > waitforselect then
				countforselect = 0
				waitforselect = 0
				if selected == 1 then
					confirmesc = false
					position = math.random(0,255)
					elapsed = math.floor(height / 5) - 1
					textatpixel = 0
					minwaittime = 4.5
					health = 3
					killed = 0
					invincible = 0
					gameovercount = 0
					score = 0
					eshots = {}
					pshots = {}
					enemy = {}
					playerx = math.floor(width / 10)
					playery = (height / 2) - 20
					gameover = false
					for x = 0, hillwidth do
						for y = 0, math.floor(height / 5) - 1 do
							hilldata:setPixel(x, y, 255,255,255,0)
						end
					end
					gamestate = 1
				elseif selected == 2 then
					selected = 1
					maxmenu = 4
					gamestate = 2
				else
					love.event.push("quit")
				end
			end
		end
	elseif gamestate == 1 then
		--game
		if waitforselect > 0 then
			countforselect = countforselect + dt
			if countforselect > waitforselect then
				scrolltext = "   "
				countforselect = 0
				waitforselect = 0
				selected = 1
				position = startpos
				elapsed = 0
				textatpixel = 0
				maxmenu = 3
				gamestate = 0
			end
		end
		if gameover then
			gameovercount = gameovercount + dt
			if gameovercount > 3.3 then
				scrolltext = "   "
				countforselect = 0
				waitforselect = 0
				selected = 1
				position = startpos
				elapsed = 0
				textatpixel = 0
				maxmenu = 3
				gamestate = 0
			end
		end
		if not paused and not confirmesc and not gameover then
			if invincible > 0 then
				invincible = invincible - dt
				if invincible < 0 then invincible = 0 end
			end
			if spacedown then
				spacecounter = spacecounter + dt
				if spacecounter > 0.5 then
					spacecounter = 0
					local nextpshot = #pshots + 1
					pshots[nextpshot] = {['x'] = playerx + 95, ['y'] = playery + 16}
					love.audio.play(sound['shoot'])
				end
			end
			if move[1] and not move[3] then
				ymove = -(height / 2) * dt
				px, py = player:getViewport()
				player:setViewport(px, 40, 120, 40)
			elseif move[3] and not move[1] then
				ymove = (height / 2) * dt
				px, py = player:getViewport()
				player:setViewport(px, 80, 120, 40)
			end
			if move[4] and not move[2] then
				xmove = -(height / 2) * dt
			elseif move[2] and not move[4] then
				xmove = (height / 2) * dt
			end
			if ((move[1] and not move[3]) or (move[3] and not move[1])) and ((move[2] and not move[4]) or (move[4] and not move[2])) then
				xmove = xmove / 1.41421356
				ymove = ymove / 1.41421356
			end
			if (move[1] and move[3]) or (not move[1] and not move[3]) then
				px, py = player:getViewport()
				player:setViewport(px, 0, 120, 40)
			end
			playerx = math.floor(playerx + xmove)
			playery = math.floor(playery + ymove)
			if playery < height / 8 then
				playery = math.floor(height / 8)
			elseif playery > height - (height / 3.3) then
				playery = height - math.floor(height / 3.3)
			end
			if playerx > width - (width / 20) then
				playerx = width - math.floor(width / 20)
			elseif playerx < width / 10 then
				playerx = math.floor(width / 10)
			end
			xmove = 0
			ymove = 0
			local rem = {}
			for z = 1, #pshots do
				pshots[z]['x'] = pshots[z]['x'] + math.floor(dt * (height * 1.4))
				if pshots[z]['x'] > width + 16 then
					table.insert(rem, z)
				else
					for a = 1, #enemy do
						if pshots[z]['x'] > enemy[a]['x'] + 16 and pshots[z]['x'] < enemy[a]['x'] + 90 and pshots[z]['y'] > enemy[a]['y'] - 1 and pshots[z]['y'] < enemy[a]['y'] + 32 and not enemy[a]['dead'] then
							table.insert(rem, z)
							enemy[a]['dead'] = true
							enemy[a]['x'] = enemy[a]['x'] + 12
							sound['explode']:setPitch(1 + (math.random() * 0.8) - 0.4)
							love.audio.play(sound['explode'])
							score = score + 5
							killed = killed + 1
							if killed == 20 then
								killed = 0
								health = health + 1
								if health > 3 then health = 3 end
							end
						end
					end
				end
			end
			for z = #rem, 1, -1 do
				table.remove(pshots, rem[z])
			end
			rem = {}
			for z = 1, #eshots do
				eshots[z]['x'] = eshots[z]['x'] - math.floor(dt * (height * 0.9))
				eshots[z]['y'] = eshots[z]['y'] + math.floor(dt * (height * eshots[z]['dir']))
				if eshots[z]['x'] < -16 or eshots[z]['y'] < -16 or eshots[z]['y'] > height then
					table.insert(rem, z)
				elseif eshots[z]['x'] < playerx + 100 and eshots[z]['x'] > playerx + 30 and eshots[z]['y'] < playery + 20 and eshots[z]['y'] > playery + 10 and invincible == 0 then
					playerhit('hit')
					table.insert(rem, z)
				end
			end
			if #rem > 0 then
				for z = #rem, 1, -1 do
					table.remove(eshots, rem[z])
				end
			end
			waitforenemy = waitforenemy + dt
			if waitforenemy > math.random(minwaittime, 10) and #enemy < width / 106 then
				waitforenemy = 0
				minwaittime = minwaittime - (minwaittime / 75)
				if minwaittime < 0.3 then minwaittime = 0.3 end
				newenemy()
			end
			for z = 1, #enemy do
				if not enemy[z]['dead'] then
					enemy[z]['timer'] = enemy[z]['timer'] + dt
					enemy[z]['x'] = math.floor(enemy[z]['x'] - 100 * dt)
					enemy[z]['y'] = math.floor(enemy[z]['y'] + math.random(-50, 50) * dt)
					if enemy[z]['x'] < -100 then
						table.insert(killenemy, z)
						score = score - 2
						if score < 0 then score = 0 end
					elseif enemy[z]['x'] > playerx - 38 and enemy[z]['x'] < playerx + 68 and enemy[z]['y'] > playery - 9 and enemy[z]['y'] < playery + 19 and invincible == 0 then
						enemy[z]['dead'] = true
						enemy[z]['x'] = enemy[z]['x'] + 12
						playerhit('explode')
					end
					if enemy[z]['y'] < height / 8 then enemy[z]['y'] = math.floor(height / 8)
					elseif enemy[z]['y'] > height - (height / 3.3) then enemy[z]['y'] = height - math.floor(height / 3.3)
					end
					if enemy[z]['timer'] > 2.5 then
						enemy[z]['timer'] = math.random()
						new = #eshots + 1
						eshots[new] = {['x'] = enemy[z]['x'] + 52, ['y'] = enemy[z]['y'] + 12, ['dir'] = math.random() - 0.5}
						sound['bloop']:setPitch(1 + (math.random() * 0.8) - 0.4)
						love.audio.play(sound['bloop'])
					end
				end
			end
			if #killenemy > 0 then
				for z = #killenemy, 1, -1 do
					table.remove(enemy, killenemy[z])
				end
			end
		else
		--pausescreen
		end
	else
		--options
		dymenu()
		if waitforselect > 0 then
			countforselect = countforselect + dt
			if countforselect > waitforselect then
				waitforselect = 0
				if selected == 2 then
					width = modes[sizeselect]['width']
					height = modes[sizeselect]['height']
					love.graphics.setMode(width, height)
					hillinit()
				elseif selected == 3 then
					if fullscreen then fullscreen = false else fullscreen = true end
					width = modes[sizeselect]['width']
					height = modes[sizeselect]['height']
					love.graphics.setMode(width, height, fullscreen)
					hillinit()
				elseif selected == 4 then
					scrolltext = "   "
					countforselect = 0
					waitforselect = 0
					selected = 1
					position = startpos
					elapsed = 0
						textatpixel = 0
					maxmenu = 3
					gamestate = 0
				end
			end
		end
	end
end

function playerhit(z)
	health = health - 1
	if z == "hit" then
		sound['hit']:setPitch(1 + (math.random() * 0.8) - 0.4)
		love.audio.play(sound['hit'])
	else
		love.audio.play(sound['explode'])
	end
	invincible = 1.7
	if health < 0 then
		gameover = true
		love.audio.play(sound['gameover'])
	end
end

function starfield(x, b)
	local starx = b or math.random(0, width)
	local stary = math.random(0, height)
	local stara = math.random(127, 255)
	local starsp = math.random(50, 70)
	local starr = (height / math.random(150, 250))
	stars[x] = {['x'] = starx, ['y'] = stary, ['alpha'] = stara, ['speed'] = starsp, ['radius'] = starr}
end

function dymenu()
	elapsed = elapsed + love.timer.getDelta()
	if elapsed >= 12 then elapsed = 0 end
	fontwidth = font:getWidth("PLANETOID") * 2
	posx = math.floor((width / 2) - fontwidth)
	posy = math.floor(height / 3) - 64
	offset = math.sin((elapsed / 3) * math.pi) * 16

	textatpixel = textatpixel + (love.timer.getDelta() * scrollspeed)
	if textatpixel > (font:getWidth(text:sub(position, position)) * 2) then
		textatpixel = 0
		position = position + 1
		if position > textlen then position = startpos end
		scrolltext = text:sub(position, position + math.floor(width / 12.8));
	end
	if (position % 100 == 0) and textatpixel == 0 then
		textcolour = math.random(1, 3)
	end
end

function logo()
	local getfont = love.graphics.getFont()
	if getfont ~= "fontgreen" or getfont ~= "fontred" or getfont ~= "fontblue" then love.graphics.setFont(fontgreen) end
	love.graphics.setFont(fontgreen)
	love.graphics.setColor(0, 0, 0, 192)
	love.graphics.print("PLANETOID", posx + 4, posy + 4 + offset, 0, 4, 4)
	if elapsed < 4 then
		love.graphics.setFont(fontgreen)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
		love.graphics.setColor(255, 255, 255, math.floor((elapsed / 4) * 255))
		love.graphics.setFont(fontblue)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
		elseif elapsed < 8 then
		love.graphics.setFont(fontblue)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
		love.graphics.setColor(255, 255, 255, math.floor(((elapsed - 4) / 4) * 255))
		love.graphics.setFont(fontred)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
		else
		love.graphics.setFont(fontred)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
		love.graphics.setColor(255, 255, 255, math.floor(((elapsed - 8) / 4) * 255))
		love.graphics.setFont(fontgreen)
		love.graphics.print("PLANETOID", posx, posy + offset, 0, 4, 4)
	end
	love.graphics.setFont(font)
	fontwidth = font:getWidth("a game by Daedalus Young")
	posx = (width / 2) - fontwidth
	for y = 1, 4 do
		love.graphics.setColor(0, 0, 0, 32 * y)
		love.graphics.print("a game by Daedalus Young", posx, posy + 40 + y, 0, 2, 1)
	end
	love.graphics.setColor(255, 255, 192, 255)
	love.graphics.print("a game by Daedalus Young", posx, posy + 45, 0, 2, 1)
end

function dyhills()
	position = position + love.timer.getDelta() * 4
	offset = offset + love.timer.getDelta()
	if offset > (updnext / 100) then
		offset = 0
		updnext = math.random(0,6)
		if updnext == 6 then
			if math.random(1,10) == 1 then
				updnext = math.random(25,80)
				dirspeed = 0
				direction = direction / 3
			end
		end
		if elapsed > (height / 5) - 9 then
			dirspeed = -3
			direction = -(math.random() * 6) + 8
		elseif elapsed < 9 then
			dirspeed = 1
			direction = (math.random() * 6) + 8
		else
			if updnext < 10 then
				direction = (math.random() * 5) + 5
				if math.random(0, 1) == 1 then
					direction = -direction
				end
			end
		end
	else
		if math.random(1, 100) == 1 then dirspeed = -dirspeed end
	end
	if dirspeed < -3 then direction = 1 elseif dirspeed > 3 then direction = -1 end
	dirspeed = dirspeed + (direction / 20)
	textatpixel = lerp(ranvalue[math.floor(position * 2)], ranvalue[math.ceil(position * 2)], (position * 2) - math.floor(position * 2))
	elapsed = elapsed + dirspeed + textatpixel
	if elapsed < 0 then elapsed = 0 elseif elapsed > height / 5 then elapsed = height / 5 end
	if position > 256 then position = 0 end
	hilldata:paste(hilldata, 0, 0, math.floor(love.timer.getDelta() * 175), 0, width, math.floor(height / 5))
	local y0 = elapsed - 6
	if y0 < 0 then y0 = 0 end
	local y1 = elapsed + 1
	if y1 > math.floor(height / 5) - 1 then y1 = math.floor(height / 5) - 1 end
	for x = width, hillwidth do
		for y = y0, y1 do
			hilldata:setPixel(x, y, 255,255,255,0)
		end
		hilldata:paste(ground, x, elapsed, math.floor(position), 0, 1, 120)
	end
	hills = love.graphics.newImage(hilldata)
	--yeah, sorry about that, it's just that Canvas is broken on my system, but doesn't look like it makes the game slow
end

function drawstars()
	for x = 1, #stars do
		love.graphics.setColor(255, (127 + (stars[x]['alpha'] * 3)) / 4, stars[x]['alpha'] / 1.5, stars[x]['alpha'] / 4)
		love.graphics.circle("fill", stars[x]['x'] + 0.5, stars[x]['y'] + 0.5, stars[x]['radius'] + 2, 6)
		love.graphics.setColor(255, (127 + (stars[x]['alpha'] * 3)) / 4, stars[x]['alpha'] / 1.5, stars[x]['alpha'] / 2)
		love.graphics.circle("fill", stars[x]['x'] + 0.5, stars[x]['y'] + 0.5, stars[x]['radius'], 6)
		love.graphics.setColor(255, (127 + (stars[x]['alpha'] * 3)) / 4, stars[x]['alpha'] / 1.5, stars[x]['alpha'])
		love.graphics.circle("fill", stars[x]['x'] + 0.5, stars[x]['y'] + 0.5, stars[x]['radius'] - 2, 6)
		if not paused and not confirmesc then
			stars[x]['x'] = stars[x]['x'] - (love.timer.getDelta() * stars[x]['speed'])
			if stars[x]['x'] < -(stars[x]['radius'] * 2) then
				starfield(x, width + (stars[x]['radius'] * 2))
			end
		end
	end
end

function updatesprites()
	spritecounter = spritecounter + love.timer.getDelta()
	killenemy = {}
	if spritecounter >= 0.1 then
		spritecounter = 0
		px, py = player:getViewport()
		px = px + 120
		if px > 360 then px = 0 end
		player:setViewport(px, py, 120, 40)
		for z = 1, #enemy do
			if not enemy[z]['dead'] then
				ex, ey = enemy[z]['sprite']:getViewport()
				ex = ex + 120
				if ex > 360 then ex = 0 end
				enemy[z]['sprite']:setViewport(ex, ey, 120, 40)
			else
				ex, ey = enemy[z]['sprite']:getViewport()
				if ey == 120 then
					ex = -96
					ey = 160
				end
				ex = ex + 96
				if ex < 480 then
					enemy[z]['sprite']:setViewport(ex, ey, 96, 40)
				else
					table.insert(killenemy, z)
				end
			end
		end
	end
end

function love.draw()
	--draw background
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(bg, 0, 0)
	drawstars()
	if gamestate == 0 then
		--menu
		--draw logo
		logo()
		--draw menu
		love.graphics.setFont(font)
		fontwidth = font:getWidth("Start Game")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 1 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Start Game", posx, (height / 2) - 30, 0, 2, 2)
		fontwidth = font:getWidth("Options")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 2 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Options", posx, (height / 2) + 30, 0, 2, 2)
		fontwidth = font:getWidth("Quit")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 3 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Quit", posx, (height / 2) + 90, 0, 2, 2)
		--draw scroll
		if textcolour == 1 then
			love.graphics.setFont(fontred)
			elseif textcolour == 2 then
			love.graphics.setFont(fontgreen)
			else love.graphics.setFont(fontblue)
		end
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print(scrolltext,  -math.floor(textatpixel + 0.5), height - 32, 0, 2, 2)

	elseif gamestate == 1 then
	--game
		--hills
		love.graphics.setColor(255, 255, 255, 255)
		if not paused and not confirmesc and not gameover then
			dyhills()
			updatesprites()
		end
		for z = 1, #pshots do
			love.graphics.drawq(bullets, plbullet, pshots[z]['x'], pshots[z]['y'])
		end
		love.graphics.draw(hills, 0, height - math.floor(height / 5))
		if (invincible > 0 and invincible < 0.1) or (invincible > 0.2 and invincible < 0.3) or (invincible > 0.4 and invincible < 0.5) or (invincible > 0.6 and invincible < 0.7) or (invincible > 0.8 and invincible < 0.9) or (invincible > 1 and invincible < 1.1) or (invincible > 1.2 and invincible < 1.3) or (invincible > 1.4 and invincible < 1.5) or (invincible > 1.6 and invincible < 1.7) or (invincible > 1.8 and invincible < 1.9) then
			love.graphics.setColor(255, 255, 255, 127)
		end
		love.graphics.drawq(sprites, player, playerx, playery)
		love.graphics.setColor(255, 255, 255, 255)
		for z = 1, #eshots do
			love.graphics.drawq(bullets, enbullet, eshots[z]['x'], eshots[z]['y'])
		end
		for z = 1, #enemy do
			love.graphics.drawq(sprites, enemy[z]['sprite'], enemy[z]['x'], enemy[z]['y'])
		end
		--score
		love.graphics.setFont(font)
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.print(tostring(score), 8, 8, 0, 4, 4)
		love.graphics.print("Health", 8, height - 28)
		love.graphics.setLineWidth(1)
		for z = 1, 3 do
			love.graphics.setColor(0, 0, 0, 127)
			love.graphics.rectangle('fill', (17 * z) - 8.5, height - 15.5, 14, 8)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.rectangle('line', (17 * z) - 8.5, height - 15.5, 14, 8)
			if health >= z then
				love.graphics.rectangle('fill', (17 * z) - 6.5, height - 13.5, 11, 5)
			end
		end
		--pause screen
		if paused then
			local fontwidth = font:getWidth("PAUSED") * 1.5
			local posx = math.floor((width / 2) - fontwidth)
			local posy = math.floor(height / 2) - 18
			love.graphics.setFont(font)
			love.graphics.setColor(0, 15, 63, 191)
			love.graphics.rectangle("fill", posx - 16, posy - 16, (fontwidth * 2) + 30, 62)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.setLineWidth(2)
			love.graphics.rectangle("line", posx - 16, posy - 16, (fontwidth * 2) + 30, 62)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.print("PAUSED", posx, posy, 0, 3, 3)
		end
		if gameover then
			local fontwidth = font:getWidth("GAME OVER") * 1.5
			local posx = math.floor((width / 2) - fontwidth)
			local posy = math.floor(height / 2) - 18
			love.graphics.setFont(font)
			love.graphics.setColor(0, 15, 63, 191)
			love.graphics.rectangle("fill", posx - 16, posy - 16, (fontwidth * 2) + 30, 62)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.setLineWidth(2)
			love.graphics.rectangle("line", posx - 16, posy - 16, (fontwidth * 2) + 30, 62)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.print("GAME OVER", posx, posy, 0, 3, 3)
		end
		if confirmesc then
			local fontwidth = font:getWidth("Quit game?") * 1.5
			local posx = math.floor((width / 2) - fontwidth)
			local posy = math.floor(height / 2) - 40
			love.graphics.setFont(font)
			love.graphics.setColor(0, 15, 63, 191)
			love.graphics.rectangle("fill", posx - 16, posy - 16, (fontwidth * 2) + 30, 107)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.setLineWidth(2)
			love.graphics.rectangle("line", posx - 16, posy - 16, (fontwidth * 2) + 30, 107)
			love.graphics.setColor(255, 255, 255, 255)
			love.graphics.print("Quit game?", posx, posy, 0, 3, 3)
			fontwidth = font:getWidth("No   Yes") * 1.5
			posx = math.floor((width / 2) - fontwidth)
			if selected == 1 then
				love.graphics.setColor(255, 255, 255, 255)
			else
				love.graphics.setColor(255, 255, 255, 127)
			end
			love.graphics.print("No", posx, posy + 45, 0, 3, 3)
			local newwidth = font:getWidth("Yes") * 3
			posx = math.floor((width / 2) + (fontwidth - newwidth))
			if selected == 2 then
				love.graphics.setColor(255, 255, 255, 255)
			else
				love.graphics.setColor(255, 255, 255, 127)
			end
			love.graphics.print("Yes", posx, posy + 45, 0, 3, 3)
		end
	else
		logo()
		--draw menu
		love.graphics.setFont(font)
		fontwidth = font:getWidth("Volume -----------")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 1 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Volume -----------", posx, (height / 2) - 30, 0, 2, 2)
		love.graphics.print("I", posx + 113 + (love.audio.getVolume() * 120), (height / 2) - 30, 0, 2, 2)
		fontwidth = font:getWidth("Screen size <          >")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 2 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Screen size", posx, (height / 2), 0, 2, 2)
		love.graphics.print(modes[sizeselect]['width'] .. " x " .. modes[sizeselect]['height'], posx + 200, (height / 2), 0, 2, 2)
		for z = 1, #modes do
			if z == sizeselect then
				if z > 1 and selected == 2 then
					love.graphics.setColor(255, 255, 255, 255)
				else
					love.graphics.setColor(255, 255, 255, 127)
				end
				love.graphics.print("<", posx + 180, (height / 2) + 4, 0, 2, 2)
				if z < #modes and selected == 2 then
					love.graphics.setColor(255, 255, 255, 255)
				else
					love.graphics.setColor(255, 255, 255, 127)
				end
				love.graphics.print(">", posx + 400, (height / 2) + 4, 0, 2, 2)
			end
		end
		if not fullscreen then fstext = "Fullscreen off" else fstext = "Fullscreen on" end
		fontwidth = font:getWidth(fstext)
		posx = math.floor((width / 2) - fontwidth)
		if selected == 3 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print(fstext, posx, (height / 2) + 30, 0, 2, 2)
		fontwidth = font:getWidth("Back")
		posx = math.floor((width / 2) - fontwidth)
		if selected == 4 then love.graphics.setColor(255, 255, 255, 255)
			else love.graphics.setColor(255, 255, 255, 127)
			end
		love.graphics.print("Back", posx, (height / 2) + 90, 0, 2, 2)
		fontwidth = font:getWidth("Space to change setting") * 0.5
		posx = math.floor((width / 2) - fontwidth)
		if selected == 2 or selected == 3 then
			love.graphics.setColor(255, 255, 255, 255)
			if selected == 2 then
				love.graphics.print("Arrow keys to select\n\nSpace to change setting", posx, (height / 2) + 135)
			else
				love.graphics.print("Space to change setting", posx, (height / 2) + 135)
			end
		end
	end

	--DEBUG
	if(showdebug) then
		love.graphics.setColor(255, 255, 255, 255)
		love.graphics.setFont(font)
		local ww, hh, fullscreen, vsync, fsaa = love.graphics.getMode()
		love.graphics.print("PLANETOID v.1.0\nFPS: " .. love.timer.getFPS() .. "\ngetDelta = " .. tostring(love.timer.getDelta()) .. "\nsize = " .. tostring(width) .. ", " .. tostring(height) .. "   fs = " .. tostring(fullscreen) .. "   vsync = " .. tostring(vsync) .. "   fsaa = " .. tostring(fsaa) .. "\ngamestate = " .. tostring(gamestate) .. "   selected = " .. tostring(selected) .. "\nmaxstars = " .. tostring(maxstars) .. "\npaused = " .. tostring(paused) .. "\nelapsed = " .. tostring(elapsed) .. "\noffset = " .. tostring(offset) .. "\npressedkey = " .. pressedkey .. "\nposition: " .. tostring(position) .. "   updnext = " .. tostring(updnext) .. "\nu " .. tostring(move[1]) .. ", r " .. tostring(move[2]) .. ", d " .. tostring(move[3]) .. ", l " .. tostring(move[4]) .. "\nminwaittime " .. tostring(minwaittime) .. "\nvolume " .. tostring(love.audio.getVolume()) .. "\nhillwidth = " .. tostring(hillwidth) .. "\npshots " .. tostring(#pshots) .. "   eshots " .. tostring(#eshots) .. "   enemy " .. tostring(#enemy) .. "\nscore " .. tostring(score) .. "   killed " .. tostring(killed) .. "   health " .. tostring(health) .. "\n\n[F3] to toggle Debug\n" .. "[ESC] to quit", 8, 8)
	end
end

function lerp(a,b,t) return a+(b-a)*t end

function love.keypressed(key)
	pressedkey = key
	if key == "f3" then
		if(showdebug) then showdebug = false else showdebug = true end
	end

	sound['select']:setPitch(1 + (math.random() * 0.2) - 0.1)
	sound['cursor']:setPitch(1 + (math.random() * 0.4) - 0.2)
	if (gamestate == 0 or gamestate == 2) and waitforselect == 0 then
		if key == "escape" then
			if selected ~= maxmenu then
				selected = maxmenu
				love.audio.play(sound['cursor'])
			else
				if gamestate == 0 then
					love.audio.play(sound['select'])
					waitforselect = 0.42
				else
					waitforselect = 0.42
					love.audio.play(sound['select'])
				end
			end
		elseif key == "down" or key == "s" then
			selected = selected + 1
			love.audio.play(sound['cursor'])
			if selected > maxmenu then selected = 1 end
		elseif key == "right" or key == "d" then
			if gamestate == 0 then
				scrollspeed = scrollspeed + 2
				if scrollspeed > 150 then scrollspeed = 150 end
			else
				if selected == 1 then
					vol = love.audio.getVolume()
					vol = vol + 0.1
					if vol > 1 then
						vol = 1
					else
						love.audio.setVolume(vol)
						love.audio.play(sound['cursor'])
					end
				elseif selected == 2 then
					sizeselect = sizeselect + 1
					if sizeselect > #modes then sizeselect = #modes end
				end
			end
		elseif key == "left" or key == "a" then
			if gamestate == 0 then
				scrollspeed = scrollspeed - 2
				if scrollspeed < 2 then scrollspeed = 2 end
			else
				if selected == 1 then
					vol = love.audio.getVolume(vol)
					vol = vol - 0.1
					if vol < 0.01 then
						vol = 0.01
					else
						love.audio.setVolume(vol)
						love.audio.play(sound['cursor'])
					end
				elseif selected == 2 then
					sizeselect = sizeselect - 1
					if sizeselect < 1 then sizeselect = 1 end
				end
			end
		elseif key == "up" or key == "w" then
			selected = selected - 1
			love.audio.play(sound['cursor'])
			if selected < 1 then selected = maxmenu end
		elseif key == " " or key == "return" then
			love.audio.play(sound['select'])
			waitforselect = 0.42
		end
	else
		sound['shoot']:setPitch(1 + (math.random() * 0.4) - 0.2)
		if key == "p" and not confirmesc then
			if paused then paused = false else paused = true end
		elseif key == "escape" then
			if not confirmesc then
				if not paused and not gameover then
					selected = 1
					confirmesc = true
				end
			end
		elseif key == " " or key == "return" then
			if not paused and not confirmesc and not gameover then
				spacedown = true
			elseif not paused and confirmesc and selected == 2 then
				love.audio.play(sound['select'])
				waitforselect = 0.2
			elseif not paused and confirmesc and selected == 1 then
				love.audio.play(sound['select'])
				confirmesc = false
			end
		elseif key == "left" or key == "a" then
			if confirmesc then
				love.audio.play(sound['cursor'])
				selected = selected - 1
				if selected < 1 then selected = 2 end
			else
				move[4] = true
			end
		elseif key == "right" or key == "d" then
			if confirmesc then
				love.audio.play(sound['cursor'])
				selected = selected + 1
				if selected > 2 then selected = 1 end
			else
				move[2] = true
			end
		elseif (key == "up" or key == "w") and not paused and not confirmesc then
			move[1] = true
		elseif (key == "down" or key == "s") and not paused and not confirmesc then
			move[3] = true
		end
	end
end

function love.keyreleased(key)
	if key == "up" or key == "w" then
		move[1] = false
	elseif key == "right" or key == "d" then
		move[2] = false
	elseif key == "down" or key == "s" then
		move[3] = false
	elseif key == "left" or key == "a" then
		move[4] = false
	elseif key == " " or key == "return" then
		spacedown = false
		spacecounter = 1
	end
end